/*

    zoltan.h - CD/MP3 Player

    Copyright (C) 2001 Angel Ortega <angel@triptico.com>

    This program is free software; you can redistribute it and/or
    modify it under the terms of the GNU General Public License
    as published by the Free Software Foundation; either version 2
    of the License, or (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.

    http://www.triptico.com

*/

#ifndef __ZOLTAN_H__
#define __ZOLTAN_H__

/* limits */
#ifndef MAX_SONGS
#define MAX_SONGS	16384
#endif

#ifndef MAX_GROUPS
#define MAX_GROUPS	512
#endif

#ifndef MAX_PLAYERS
#define MAX_PLAYERS	32
#endif

#ifndef SONG_CACHE_SIZE
#define SONG_CACHE_SIZE  65536
#endif

/* CD types */
#define TYPE_UNKNOWN	-1
#define TYPE_CDROM	0
#define TYPE_CDA	1
#define TYPE_HD 	2

#ifdef __cplusplus
extern "C"
{
#endif

/* a song */
struct song
{
	char * filename;
	char * songinfo;
	int group;
	int length;
	int pos;
	int player;
};

extern struct song songs[MAX_SONGS];

/* a group */
struct group
{
	char * groupinfo;
};

extern struct group groups[MAX_GROUPS];

extern int _num_songs;
extern int _num_groups;
extern int _media_type;
extern int _cda_total_time;
extern int _cda_num_tracks;
extern int _cddb_id;
extern int _current_song;

extern char _cd_device[128];
extern char _mixer_device[128];
extern char _mount_point[128];
extern char _cddb_cache[128];
extern char _hd_song_dir[128];
extern char _cddb_host[512];
extern char _cddb_cgi[512];
extern int _zoltan_port;
extern int _zoltan_volume;
extern int _zoltan_volume_increment;
extern int _zoltan_paused;

int zoltan_mount(void);
void zoltan_umount(void);
void zoltan_hd(void);
void zoltan_play(void);
void zoltan_stop(void);
void zoltan_pause(void);
void zoltan_change_group(int inc);
char * zoltan_build_song_info(void);
void zoltan_poll(void);
void zoltan_set_volume(void);
int zoltan_time_left(void);
void zoltan_cache_store_song(void);
int zoltan_search_cache(char * songinfo);
void zoltan_load_song_cache(void);
void zoltan_save_song_cache(void);

int zoltan_cd_test(void);
int zoltan_cda_info(void);
void zoltan_add_song(char * songinfo, char * filename, int group, int player);
void zoltan_add_group(char * groupinfo);
void zoltan_song_find(char * root, char * directory, int group);
void zoltan_add_player(char * ext, char * path);

void zoltan_write(int a, char * str);
void zoltan_writef(int a, char * fmt, ...);
int zoltan_client_socket(char * server, int port);
void zoltan_send_http_query(int s, char * query, char * host);
void zoltan_drop_http_response(int s);
int zoltan_cddb_query(void);

int zoltan_server_socket(int port);
int zoltan_accept(int s);

void zoltan_process_connection(int s);

int zoltan_startup(char * configfile);

#ifdef __cplusplus
}
#endif

#endif /* __ZOLTAN_H__ */
