/*

    zoltan_server.c - ZoltanPlayer CD/MP3 Player

    Server code (httpd).

    Copyright (C) 2001 Angel Ortega <angel@triptico.com>

    This program is free software; you can redistribute it and/or
    modify it under the terms of the GNU General Public License
    as published by the Free Software Foundation; either version 2
    of the License, or (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.

    http://www.triptico.com

*/


#include <stdio.h>
#include <string.h>
#include <stdlib.h>
#include <unistd.h>
#include <ctype.h>
#include <time.h>

#include "zoltan.h"


/*******************
	Data
********************/

int t;

/*******************
	Code
********************/

static void zoltan_html_controls(int a)
{
	zoltan_write(a,"<!-- control panel -->\n");

	zoltan_write(a,"<table border=1><tr align=center>\n");

	if(_current_song != -1)
	{
		zoltan_write(a,"<td width='25%'><small><a href='/stop'>Stop</a></small></td>");
		zoltan_writef(a,
			"<td width='25%'><small><a href='/pause/%d#C'>%s</a></small></td>",
			t, _zoltan_paused ? "Unpause" : "Pause");
	}
	else
		zoltan_write(a,"<td width='25%'><small>Stop</small></td><td width='25%'><small>Pause</small></td>");

	if(_media_type==TYPE_HD)
	{
		zoltan_write(a,"<td width='25%'><small><a href='/mount'>Load CD</a></small></td>");
		zoltan_write(a,"<td width='25%'><small><a href='/hd'>Re-read HD</a></small></td>");
	}
	else
	{
		zoltan_write(a,"<td width='25%'><small><a href='/umount'>Unload CD</a></small></td>");

		 if(_hd_song_dir[0]!='\0')
			zoltan_write(a,"<td width='25%'><small><a href='/hd'>HD</a></small></td>");
		else
			zoltan_write(a,"<td width='25%'><small>HD</small></td>");
	}

	zoltan_write(a,"\n<tr align=center>");

	if(_num_groups > 1)
		zoltan_writef(a,"<td colspan=2><small><a href='/prev-group/%d#C'>[&lt;]</a> Group <a href='/next-group%d#C'>[&gt;]</a></small></td>",t,t);
	else
		zoltan_write(a,"<td colspan=2><small>[&lt;] Group [&gt;]</small></td>");

	zoltan_writef(a,"<td colspan=2><small><a href='/prev-song/%d#C'>[&lt;]</a> Song <a href='/next-song/%d#C'>[&gt;]</a></small></td>",t,t);

	if(_mixer_device[0] != '\0')
	{
		zoltan_write(a,"\n<tr align=center>");

		zoltan_writef(a,
		"<td colspan=4><small><a href='/vol-down/%d#C'>[&lt;]</a> Vol (%d%%) <a href='/vol-up/%d#C'>[&gt;]</a></small></td>",
		t, _zoltan_volume, t);
	}

	zoltan_write(a,"</table><p>\n");
}


/**
 * zoltan_html_page - Shows the Zoltan html page
 * @a: socket
 * @read_only: flag for generating a static page
 *
 * Generates a html page with current zoltan status
 * (current song info, available songs, etc.) and
 * writes into the socket. If read_only is not set,
 * the page contains clickable references for
 * directly playing songs and some navigation links.
 */
static void zoltan_html_page(int a, int read_only)
{
	char * ptr;
	int n,g,l;

	t=(int) time(NULL);

	if(!read_only && _current_song!=-1)
	{
		l=zoltan_time_left();

		zoltan_writef(a,"<meta http-equiv='Refresh' content='%d; URL=/%d#C'>", l, t);
	}

	zoltan_write(a,"<head><title>ZoltanPlayer " VERSION "</title>\n");
	zoltan_write(a,
	"<body bgcolor=white text=black link='#0000aa' alink='#0000aa' vlink='#0000aa'>\n");

	if(_current_song==-1) zoltan_write(a,"<a name='C'></a>");

	if(read_only)
		zoltan_write(a,"<h1>ZoltanPlayer " VERSION "</h1>");
	else
		zoltan_writef(a,"<a href='/%d#C'><h1>ZoltanPlayer " VERSION "</h1></a>", t);

	if(_media_type==TYPE_UNKNOWN)
	{
		if(read_only)
			zoltan_write(a,"No songs to play<br>");
		else
		{
			zoltan_write(a,"<a href='/mount'>Load CD</a>");

			if(_hd_song_dir[0]!='\0')
				zoltan_write(a," | <a href='/hd'>HD</a>\n");
		}

		return;
	}

	if(_media_type==TYPE_CDA)
		zoltan_writef(a,"\n<!-- cddb id: %04x -->\n",_cddb_id);

	if(_current_song==-1)
		zoltan_write(a,"<b>Idle</b><p>\n");
	else
	{
/*		  zoltan_write(a,"<h3><font color='#00aa00'>\n");

		if((ptr=groups[songs[_current_song].group].groupinfo)!=NULL && *ptr!='\0')
			zoltan_writef(a,"<a href='#grp-%d'>%s</a><br>\n",
				songs[_current_song].group,ptr);

		zoltan_writef(a,"%d - <em>%s</em><br></font></h3>\n", _current_song+1,
			songs[_current_song].songinfo);
*/
		if(_zoltan_paused)
			zoltan_write(a,"<b>Paused</b><p>\n");
	}

	for(g=-1,n=0;n < _num_songs;n++)
	{
		if(songs[n].group != g)
		{
			if(n!=0) zoltan_write(a,"</table>\n");

			if((ptr=groups[songs[n].group].groupinfo)!=NULL && *ptr!='\0')
				zoltan_writef(a,"<a name='grp-%d'></a><h2>%s</h2>\n",songs[n].group,ptr);

			/* controls */
			if(!read_only && _current_song==-1 && g==-1)
				zoltan_html_controls(a);

			g=songs[n].group;

			zoltan_write(a,"<table width='100%'>\n");
		}

		if(n == _current_song)
			zoltan_writef(a,"<tr><td valign=top bgcolor='#0000aa'><a name='C'></a><font color='#ffffff'><em><b> %d - ", n+1);
		else
		if(read_only)
			zoltan_writef(a,"<tr><td bgcolor='%s' colspan=2><font color='black'> %d - ",
				n&1 ? "#f0f0f0" : "#e0e0e0", n+1);
		else
			zoltan_writef(a,"<tr><td bgcolor='%s' colspan=2><font color='black'> %d - <a href='/goto-%d/%d#C'>",
				n&1 ? "#f0f0f0" : "#e0e0e0", n+1, n+1, t);

		if(songs[n].length)
			zoltan_writef(a,"%s [%02d:%02d]",
				songs[n].songinfo,
				songs[n].length / 60,
				songs[n].length % 60);
		else
			zoltan_write(a,songs[n].songinfo);

		if(n == _current_song && !read_only)
		{
			zoltan_write(a,"</td><td width='25%'>");
			zoltan_html_controls(a);
		}

		zoltan_write(a,"</td></tr>\n");
	}

	zoltan_write(a,"</table>\n");
}


/**
 * zoltan_command - Process a command
 * @a: the connection socket
 * @cmd: the command to be processed
 *
 * Processes a command, received from an HTTP connection.
 * The correspondent zoltan command is called, and
 * the html page is generated.
 */
static void zoltan_command(int a, char * cmd)
{
	int play=0;
	int read_only=0;
	int page=1;
	int n;

	if(strcmp(cmd,"mount")==0)
	{
		zoltan_mount();
	}
	else
	if(strcmp(cmd,"umount")==0)
	{
		zoltan_umount();
	}
	else
	if(strcmp(cmd,"xmount")==0)
	{
		if(_media_type==TYPE_UNKNOWN)
			zoltan_mount();
		else
			zoltan_umount();
	}
	else
	if(strcmp(cmd,"pause")==0)
	{
		zoltan_pause();
	}
	else
	if(strcmp(cmd,"hd")==0)
	{
		zoltan_hd();
	}
	else
	if(strcmp(cmd,"play")==0)
	{
		_current_song=0;
		play=1;
	}
	else
	if(strcmp(cmd,"stop")==0)
	{
		zoltan_stop();
		_current_song=-1;
	}
	else
	if(strcmp(cmd,"next-song")==0)
	{
		if(_current_song < _num_songs-1)
		{
			_current_song++;
			play=1;
		}
	}
	else
	if(strcmp(cmd,"prev-song")==0)
	{
		if(_current_song > 0)
		{
			_current_song--;
			play=1;
		}
	}
	else
	if(strncmp(cmd,"goto-",5)==0)
	{
		n=atoi(cmd+5)-1;

		if(n >= 0 && n < _num_songs)
		{
			_current_song=n;
			play=1;
		}
	}
	else
	if(strcmp(cmd,"next-group")==0)
	{
		zoltan_change_group(1);
		play=1;
	}
	else
	if(strcmp(cmd,"prev-group")==0)
	{
		zoltan_change_group(-1);
		play=1;
	}
	else
	if(strcmp(cmd,"song-info")==0)
	{
		if(_current_song!=-1)
		{
			zoltan_write(a, zoltan_build_song_info());
			zoltan_write(a,"\n");
		}

		page=0;
	}
	else
	if(strcmp(cmd,"vol-up")==0)
	{
		if(_zoltan_volume < 100)
			_zoltan_volume+=_zoltan_volume_increment;

		zoltan_set_volume();
	}
	else
	if(strcmp(cmd,"vol-down")==0)
	{
		if(_zoltan_volume > 0)
			_zoltan_volume-=_zoltan_volume_increment;

		zoltan_set_volume();
	}
	else
	if(strcmp(cmd,"cd-info")==0)
		read_only=1;

	if(page) zoltan_html_page(a, read_only);

	close(a);
	if(play) zoltan_play();
}


/**
 * zoltan_process_connection - Process an incoming connection
 * @s: the socket
 *
 * Processes a complete incoming HTTP connection, calling
 * zoltan_command() if applicable.
 */
void zoltan_process_connection(int s)
{
	int a,n;
	unsigned char c;
	char cmd[32];

	/* let's be extremely careful and conservative */

	if((a=zoltan_accept(s)) <= 0)
		return;

	/* read first GET/ */
	if(read(a,cmd,5)!=5) goto hell;
	if(memcmp(cmd,"GET /",5)!=0) goto hell;

	/* read command */
	for(n=0;n < sizeof(cmd) - 1;)
	{
		if(read(a,&c,1)!=1) goto hell;
		if(!isalnum(c) && c!='-')
			break;

		cmd[n++]=c;
	}
	cmd[n]='\0';

	printf("cmd: %s\n",cmd);

	/* read the rest of client input as lines,
	   until a blank one is found */
	n=0;
	for(;;)
	{
		if(read(a,&c,1)!=1) goto hell;

		if(c=='\n')
		{
			if(n==0) break;
			n=0;
		}
		else
		if(c!='\r')
			n++;
	}

	write(a,"HTTP/1.0 200 OK\r\n",17);
	write(a,"Cache-control: no-cache\r\n",25);
	write(a,"Pragma: no-cache\r\n",18);
	write(a,"Content-Type: text/html\r\n\r\n",27);

	zoltan_command(a, cmd);

hell:
	close(a);
}
